@@ -1,9 +1,9 @@
-/* $Id: svgamesa.h,v 1.5 1997/09/29 23:22:16 brianp Exp $ */
+/* $Id: svgamesa.h,v 1.4 2000/02/09 19:03:28 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.5
- * Copyright (C) 1995-1997  Brian Paul
+ * Version:  3.3
+ * Copyright (C) 1995-2000  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,27 +21,6 @@
  */
 
 
-/*
- * $Log: svgamesa.h,v $
- * Revision 1.5  1997/09/29 23:22:16  brianp
- * updated version to 2.5
- *
- * Revision 1.4  1997/07/11 02:55:24  brianp
- * added doubleBuffer flag to SVGAMesaCreateContext, bumped version to 2.4
- *
- * Revision 1.3  1996/10/30 03:12:47  brianp
- * incremented version to 2.1
- *
- * Revision 1.2  1996/10/15 00:26:02  brianp
- * added prototype for SVGAMesaSwapBuffers()
- *
- * Revision 1.1  1996/09/13 01:26:41  brianp
- * Initial revision
- *
- */
-
-
-
 /*
  * SVGA/Mesa interface for Linux.
  */
@@ -73,8 +52,8 @@
 #define SVGAMESA_H
 
 
-#define SVGAMESA_MAJOR_VERSION 2
-#define SVGAMESA_MINOR_VERSION 5
+#define SVGAMESA_MAJOR_VERSION 3
+#define SVGAMESA_MINOR_VERSION 3
 
 
 #ifdef __cplusplus
@@ -96,16 +75,21 @@ typedef struct svgamesa_context *SVGAMesaContext;
 /*
  * doubleBuffer flag new in version 2.4
  */
+extern int SVGAMesaInit( int GraphMode );
+
+extern int SVGAMesaClose( void );
+
 extern SVGAMesaContext SVGAMesaCreateContext( GLboolean doubleBuffer );
 
 extern void SVGAMesaDestroyContext( SVGAMesaContext ctx );
 
 extern void SVGAMesaMakeCurrent( SVGAMesaContext ctx );
 
-extern SVGAMesaContext SVGAMesaGetCurrentContext( void );
-
 extern void SVGAMesaSwapBuffers( void );
 
+extern void SVGAMesaSetCI(int ndx, GLubyte red, GLubyte green, GLubyte blue);
+
+extern SVGAMesaContext SVGAMesaGetCurrentContext( void );
 
 #ifdef __cplusplus
 }
