@@ -1,10 +1,10 @@
-/* $Id: tritemp.h,v 1.2.2.1 2000/06/18 22:48:20 brianp Exp $ */
+/* $Id: tritemp.h,v 1.8 2000/06/18 22:48:27 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.1
+ * Version:  3.3
  * 
- * Copyright (C) 1999  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2000  Brian Paul   All Rights Reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -51,6 +51,10 @@
  *    PIXEL_ADDRESS(X,Y)  - returns the address of pixel at (X,Y) where
  *                          Y==0 at bottom of screen and increases upward.
  *
+ * Similarly, for direct depth buffer access, this type is used for depth
+ * buffer addressing:
+ *    DEPTH_TYPE          - either GLushort or GLuint
+ *
  * Optionally, one may provide one-time setup code per triangle:
  *    SETUP_CODE    - code which is to be executed once per triangle
  * 
@@ -83,7 +87,13 @@
 	GLfixed fx0;	/* fixed pt X of lower endpoint */
    } EdgeT;
 
-   struct vertex_buffer *VB = ctx->VB;
+#ifdef INTERP_Z
+   const GLint depthBits = ctx->Visual->DepthBits;
+   const GLint fixedToDepthShift = depthBits <= 16 ? FIXED_SHIFT : 0;
+   const GLfloat maxDepth = ctx->Visual->DepthMaxF;
+#define FixedToDepth(F)  ((F) >> fixedToDepthShift)
+#endif
+   const struct vertex_buffer *VB = ctx->VB;
    EdgeT eMaj, eTop, eBot;
    GLfloat oneOverArea;
    int vMin, vMid, vMax;       /* vertex indexes:  Y(vMin)<=Y(vMid)<=Y(vMax) */
@@ -134,7 +144,7 @@
 
    /* compute oneOverArea */
    {
-      GLfloat area = eMaj.dx * eBot.dy - eBot.dx * eMaj.dy;
+      const GLfloat area = eMaj.dx * eBot.dy - eBot.dx * eMaj.dy;
 
       /* Do backface culling */
       if (area * bf < 0.0)
@@ -150,6 +160,10 @@
          oneOverArea = 1.0F / area;
    }
 
+#ifndef DO_OCCLUSION_TEST
+   ctx->OcclusionResult = GL_TRUE;
+#endif
+
    /* Edge setup.  For a triangle strip these could be reused... */
    {
       /* fixed point Y coordinates */
@@ -229,36 +243,36 @@
 
    {
       GLint ltor;		/* true if scanning left-to-right */
-#if INTERP_Z
+#ifdef INTERP_Z
       GLfloat dzdx, dzdy;      GLfixed fdzdx;
 #endif
-#if INTERP_RGB
+#ifdef INTERP_RGB
       GLfloat drdx, drdy;      GLfixed fdrdx;
       GLfloat dgdx, dgdy;      GLfixed fdgdx;
       GLfloat dbdx, dbdy;      GLfixed fdbdx;
 #endif
-#if INTERP_SPEC
+#ifdef INTERP_SPEC
       GLfloat dsrdx, dsrdy;    GLfixed fdsrdx;
       GLfloat dsgdx, dsgdy;    GLfixed fdsgdx;
       GLfloat dsbdx, dsbdy;    GLfixed fdsbdx;
 #endif
-#if INTERP_ALPHA
+#ifdef INTERP_ALPHA
       GLfloat dadx, dady;      GLfixed fdadx;
 #endif
-#if INTERP_INDEX
+#ifdef INTERP_INDEX
       GLfloat didx, didy;      GLfixed fdidx;
 #endif
-#if INTERP_INT_ST
+#ifdef INTERP_INT_ST
       GLfloat dsdx, dsdy;      GLfixed fdsdx;
       GLfloat dtdx, dtdy;      GLfixed fdtdx;
 #endif
-#if INTERP_STUV
+#ifdef INTERP_STUV
       GLfloat dsdx, dsdy;
       GLfloat dtdx, dtdy;
       GLfloat dudx, dudy;
       GLfloat dvdx, dvdy;
 #endif
-#if INTERP_STUV1
+#ifdef INTERP_STUV1
       GLfloat ds1dx, ds1dy;
       GLfloat dt1dx, dt1dy;
       GLfloat du1dx, du1dy;
@@ -275,13 +289,13 @@
       ltor = (oneOverArea < 0.0F);
 
       /* compute d?/dx and d?/dy derivatives */
-#if INTERP_Z
+#ifdef INTERP_Z
       {
          GLfloat eMaj_dz, eBot_dz;
          eMaj_dz = VB->Win.data[vMax][2] - VB->Win.data[vMin][2];
          eBot_dz = VB->Win.data[vMid][2] - VB->Win.data[vMin][2];
          dzdx = oneOverArea * (eMaj_dz * eBot.dy - eMaj.dy * eBot_dz);
-         if (dzdx>DEPTH_SCALE || dzdx<-DEPTH_SCALE) {
+         if (dzdx > maxDepth || dzdx < -maxDepth) {
             /* probably a sliver triangle */
             dzdx = 0.0;
             dzdy = 0.0;
@@ -289,14 +303,13 @@
          else {
             dzdy = oneOverArea * (eMaj.dx * eBot_dz - eMaj_dz * eBot.dx);
          }
-#if DEPTH_BITS==16
-         fdzdx = SignedFloatToFixed(dzdx);
-#else
-         fdzdx = (GLint) dzdx;
-#endif
+         if (depthBits <= 16)
+            fdzdx = SignedFloatToFixed(dzdx);
+         else
+            fdzdx = (GLint) dzdx;
       }
 #endif
-#if INTERP_RGB
+#ifdef INTERP_RGB
       {
          GLfloat eMaj_dr, eBot_dr;
          eMaj_dr = (GLint) VB->ColorPtr->data[vMax][0] - (GLint) VB->ColorPtr->data[vMin][0];
@@ -322,7 +335,7 @@
 	 dbdy = oneOverArea * (eMaj.dx * eBot_db - eMaj_db * eBot.dx);
       }
 #endif
-#if INTERP_SPEC
+#ifdef INTERP_SPEC
       {
          GLfloat eMaj_dsr, eBot_dsr;
          eMaj_dsr = (GLint) VB->Specular[vMax][0] - (GLint) VB->Specular[vMin][0];
@@ -348,7 +361,7 @@
 	 dsbdy = oneOverArea * (eMaj.dx * eBot_dsb - eMaj_dsb * eBot.dx);
       }
 #endif
-#if INTERP_ALPHA
+#ifdef INTERP_ALPHA
       {
          GLfloat eMaj_da, eBot_da;
          eMaj_da = (GLint) VB->ColorPtr->data[vMax][3] - (GLint) VB->ColorPtr->data[vMin][3];
@@ -358,7 +371,7 @@
          dady = oneOverArea * (eMaj.dx * eBot_da - eMaj_da * eBot.dx);
       }
 #endif
-#if INTERP_INDEX
+#ifdef INTERP_INDEX
       {
          GLfloat eMaj_di, eBot_di;
          eMaj_di = (GLint) VB->IndexPtr->data[vMax] - (GLint) VB->IndexPtr->data[vMin];
@@ -368,7 +381,7 @@
          didy = oneOverArea * (eMaj.dx * eBot_di - eMaj_di * eBot.dx);
       }
 #endif
-#if INTERP_INT_ST
+#ifdef INTERP_INT_ST
       {
          GLfloat eMaj_ds, eBot_ds;
          eMaj_ds = (VB->TexCoordPtr[0]->data[vMax][0] - VB->TexCoordPtr[0]->data[vMin][0]) * S_SCALE;
@@ -392,7 +405,7 @@
       }
 
 #endif
-#if INTERP_STUV
+#ifdef INTERP_STUV
       {
          GLfloat wMax = VB->Win.data[vMax][3];
          GLfloat wMin = VB->Win.data[vMin][3];
@@ -444,7 +457,7 @@
 	 }
       }
 #endif
-#if INTERP_STUV1
+#ifdef INTERP_STUV1
       {
          GLfloat wMax = VB->Win.data[vMax][3];
          GLfloat wMin = VB->Win.data[vMin][3];
@@ -562,38 +575,40 @@
          PIXEL_TYPE *pRow;
          int dPRowOuter, dPRowInner;  /* offset in bytes */
 #endif
-#if INTERP_Z
-         GLdepth *zRow;
+#ifdef INTERP_Z
+#  ifdef DEPTH_TYPE
+         DEPTH_TYPE *zRow;
          int dZRowOuter, dZRowInner;  /* offset in bytes */
+#  endif
          GLfixed fz, fdzOuter, fdzInner;
 #endif
-#if INTERP_RGB
+#ifdef INTERP_RGB
          GLfixed fr, fdrOuter, fdrInner;
          GLfixed fg, fdgOuter, fdgInner;
          GLfixed fb, fdbOuter, fdbInner;
 #endif
-#if INTERP_SPEC
+#ifdef INTERP_SPEC
          GLfixed fsr, fdsrOuter, fdsrInner;
          GLfixed fsg, fdsgOuter, fdsgInner;
          GLfixed fsb, fdsbOuter, fdsbInner;
 #endif
-#if INTERP_ALPHA
+#ifdef INTERP_ALPHA
          GLfixed fa, fdaOuter, fdaInner;
 #endif
-#if INTERP_INDEX
+#ifdef INTERP_INDEX
          GLfixed fi, fdiOuter, fdiInner;
 #endif
-#if INTERP_INT_ST
+#ifdef INTERP_INT_ST
          GLfixed fs, fdsOuter, fdsInner;
          GLfixed ft, fdtOuter, fdtInner;
 #endif
-#if INTERP_STUV
+#ifdef INTERP_STUV
          GLfloat sLeft, dsOuter, dsInner;
          GLfloat tLeft, dtOuter, dtInner;
          GLfloat uLeft, duOuter, duInner;
          GLfloat vLeft, dvOuter, dvInner;
 #endif
-#if INTERP_STUV1
+#ifdef INTERP_STUV1
          GLfloat s1Left, ds1Outer, ds1Inner;
          GLfloat t1Left, dt1Outer, dt1Inner;
          GLfloat u1Left, du1Outer, du1Inner;
@@ -682,29 +697,30 @@
                 * pixel that's actually inside the triangle.
                 */
 
-#if INTERP_Z
+#ifdef INTERP_Z
                {
-                  GLfloat z0, tmp;
-                  z0 = VB->Win.data[vLower][2] + ctx->PolygonZoffset;
-#if DEPTH_BITS==16
-                  /* interpolate fixed-pt values */
-                  tmp = (z0 * FIXED_SCALE + dzdx * adjx + dzdy * adjy) + FIXED_HALF;
-                  if (tmp < MAX_GLUINT/2)
-                     fz = (GLfixed) tmp;
-                  else
-                     fz = MAX_GLUINT/2;
-                  fdzOuter = SignedFloatToFixed(dzdy + dxOuter * dzdx);
-#else
-                  (void) tmp;
-                  /* interpolate depth values exactly */
-                  fz = (GLint) (z0 + dzdx*FixedToFloat(adjx) + dzdy*FixedToFloat(adjy));
-                  fdzOuter = (GLint) (dzdy + dxOuter * dzdx);
-#endif
-                  zRow = Z_ADDRESS( ctx, FixedToInt(fxLeftEdge), iy );
-                  dZRowOuter = (ctx->Buffer->Width + idxOuter) * sizeof(GLdepth);
+                  GLfloat z0 = VB->Win.data[vLower][2] + ctx->PolygonZoffset;
+                  if (depthBits <= 16) {
+                     /* interpolate fixed-pt values */
+                     GLfloat tmp = (z0 * FIXED_SCALE + dzdx * adjx + dzdy * adjy) + FIXED_HALF;
+                     if (tmp < MAX_GLUINT / 2)
+                        fz = (GLfixed) tmp;
+                     else
+                        fz = MAX_GLUINT / 2;
+                     fdzOuter = SignedFloatToFixed(dzdy + dxOuter * dzdx);
+                  }
+                  else {
+                     /* interpolate depth values exactly */
+                     fz = (GLint) (z0 + dzdx*FixedToFloat(adjx) + dzdy*FixedToFloat(adjy));
+                     fdzOuter = (GLint) (dzdy + dxOuter * dzdx);
+                  }
+#  ifdef DEPTH_TYPE
+                  zRow = (DEPTH_TYPE *) _mesa_zbuffer_address(ctx, FixedToInt(fxLeftEdge), iy);
+                  dZRowOuter = (ctx->DrawBuffer->Width + idxOuter) * sizeof(DEPTH_TYPE);
+#  endif
                }
 #endif
-#if INTERP_RGB
+#ifdef INTERP_RGB
                fr = (GLfixed)(IntToFixed(VB->ColorPtr->data[vLower][0]) + drdx * adjx + drdy * adjy)
                     + FIXED_HALF;
                fdrOuter = SignedFloatToFixed(drdy + dxOuter * drdx);
@@ -717,7 +733,7 @@
                     + FIXED_HALF;
                fdbOuter = SignedFloatToFixed(dbdy + dxOuter * dbdx);
 #endif
-#if INTERP_SPEC
+#ifdef INTERP_SPEC
                fsr = (GLfixed)(IntToFixed(VB->Specular[vLower][0]) + dsrdx * adjx + dsrdy * adjy)
                     + FIXED_HALF;
                fdsrOuter = SignedFloatToFixed(dsrdy + dxOuter * dsrdx);
@@ -730,17 +746,17 @@
                     + FIXED_HALF;
                fdsbOuter = SignedFloatToFixed(dsbdy + dxOuter * dsbdx);
 #endif
-#if INTERP_ALPHA
+#ifdef INTERP_ALPHA
                fa = (GLfixed)(IntToFixed(VB->ColorPtr->data[vLower][3]) + dadx * adjx + dady * adjy)
                     + FIXED_HALF;
                fdaOuter = SignedFloatToFixed(dady + dxOuter * dadx);
 #endif
-#if INTERP_INDEX
+#ifdef INTERP_INDEX
                fi = (GLfixed)(VB->IndexPtr->data[vLower] * FIXED_SCALE + didx * adjx
                               + didy * adjy) + FIXED_HALF;
                fdiOuter = SignedFloatToFixed(didy + dxOuter * didx);
 #endif
-#if INTERP_INT_ST
+#ifdef INTERP_INT_ST
                {
                   GLfloat s0, t0;
                   s0 = VB->TexCoordPtr[0]->data[vLower][0] * S_SCALE;
@@ -761,7 +777,7 @@
 		  }
 	       }
 #endif
-#if INTERP_STUV
+#ifdef INTERP_STUV
                {
                   GLfloat invW = VB->Win.data[vLower][3];
                   GLfloat s0, t0, u0, v0;
@@ -794,7 +810,7 @@
 		  dvOuter = dvdy + dxOuter * dvdx;
                }
 #endif
-#if INTERP_STUV1
+#ifdef INTERP_STUV1
                {
                   GLfloat invW = VB->Win.data[vLower][3];
                   GLfloat s0, t0, u0, v0;
@@ -845,37 +861,39 @@
 #ifdef PIXEL_ADDRESS
             dPRowInner = dPRowOuter + sizeof(PIXEL_TYPE);
 #endif
-#if INTERP_Z
-            dZRowInner = dZRowOuter + sizeof(GLdepth);
+#ifdef INTERP_Z
+#  ifdef DEPTH_TYPE
+            dZRowInner = dZRowOuter + sizeof(DEPTH_TYPE);
+#  endif
             fdzInner = fdzOuter + fdzdx;
 #endif
-#if INTERP_RGB
+#ifdef INTERP_RGB
             fdrInner = fdrOuter + fdrdx;
             fdgInner = fdgOuter + fdgdx;
             fdbInner = fdbOuter + fdbdx;
 #endif
-#if INTERP_SPEC
+#ifdef INTERP_SPEC
             fdsrInner = fdsrOuter + fdsrdx;
             fdsgInner = fdsgOuter + fdsgdx;
             fdsbInner = fdsbOuter + fdsbdx;
 #endif
-#if INTERP_ALPHA
+#ifdef INTERP_ALPHA
             fdaInner = fdaOuter + fdadx;
 #endif
-#if INTERP_INDEX
+#ifdef INTERP_INDEX
             fdiInner = fdiOuter + fdidx;
 #endif
-#if INTERP_INT_ST
+#ifdef INTERP_INT_ST
             fdsInner = fdsOuter + fdsdx;
             fdtInner = fdtOuter + fdtdx;
 #endif
-#if INTERP_STUV
+#ifdef INTERP_STUV
 	    dsInner = dsOuter + dsdx;
 	    dtInner = dtOuter + dtdx;
 	    duInner = duOuter + dudx;
 	    dvInner = dvOuter + dvdx;
 #endif
-#if INTERP_STUV1
+#ifdef INTERP_STUV1
 	    ds1Inner = ds1Outer + ds1dx;
 	    dt1Inner = dt1Outer + dt1dx;
 	    du1Inner = du1Outer + du1dx;
@@ -885,35 +903,34 @@
             while (lines>0) {
                /* initialize the span interpolants to the leftmost value */
                /* ff = fixed-pt fragment */
-#if INTERP_Z
+#ifdef INTERP_Z
                GLfixed ffz = fz;
-               /*GLdepth *zp = zRow;*/
 #endif
-#if INTERP_RGB
+#ifdef INTERP_RGB
                GLfixed ffr = fr,  ffg = fg,  ffb = fb;
 #endif
-#if INTERP_SPEC
+#ifdef INTERP_SPEC
                GLfixed ffsr = fsr,  ffsg = fsg,  ffsb = fsb;
 #endif
-#if INTERP_ALPHA
+#ifdef INTERP_ALPHA
                GLfixed ffa = fa;
 #endif
-#if INTERP_INDEX
+#ifdef INTERP_INDEX
                GLfixed ffi = fi;
 #endif
-#if INTERP_INT_ST
+#ifdef INTERP_INT_ST
                GLfixed ffs = fs,  fft = ft;
 #endif
-#if INTERP_STUV
+#ifdef INTERP_STUV
                GLfloat ss = sLeft, tt = tLeft, uu = uLeft, vv = vLeft;
 #endif
-#if INTERP_STUV1
+#ifdef INTERP_STUV1
                GLfloat ss1 = s1Left, tt1 = t1Left, uu1 = u1Left, vv1 = v1Left;
 #endif
                GLint left = FixedToInt(fxLeftEdge);
                GLint right = FixedToInt(fxRightEdge);
 
-#if INTERP_RGB
+#ifdef INTERP_RGB
                {
                   /* need this to accomodate round-off errors */
                   GLfixed ffrend = ffr+(right-left-1)*fdrdx;
@@ -927,7 +944,7 @@
                   if (ffb<0) ffb = 0;
                }
 #endif
-#if INTERP_SPEC
+#ifdef INTERP_SPEC
                {
                   /* need this to accomodate round-off errors */
                   GLfixed ffsrend = ffsr+(right-left-1)*fdsrdx;
@@ -941,14 +958,14 @@
                   if (ffsb<0) ffsb = 0;
                }
 #endif
-#if INTERP_ALPHA
+#ifdef INTERP_ALPHA
                {
                   GLfixed ffaend = ffa+(right-left-1)*fdadx;
                   if (ffaend<0) ffa -= ffaend;
                   if (ffa<0) ffa = 0;
                }
 #endif
-#if INTERP_INDEX
+#ifdef INTERP_INDEX
                if (ffi<0) ffi = 0;
 #endif
 
@@ -971,34 +988,36 @@
                if (fError >= 0) {
                   fError -= FIXED_ONE;
 #ifdef PIXEL_ADDRESS
-                  pRow = (PIXEL_TYPE*) ((GLubyte*)pRow + dPRowOuter);
+                  pRow = (PIXEL_TYPE *) ((GLubyte*)pRow + dPRowOuter);
 #endif
-#if INTERP_Z
-                  zRow = (GLdepth*) ((GLubyte*)zRow + dZRowOuter);
+#ifdef INTERP_Z
+#  ifdef DEPTH_TYPE
+                  zRow = (DEPTH_TYPE *) ((GLubyte*)zRow + dZRowOuter);
+#  endif
                   fz += fdzOuter;
 #endif
-#if INTERP_RGB
+#ifdef INTERP_RGB
                   fr += fdrOuter;   fg += fdgOuter;   fb += fdbOuter;
 #endif
-#if INTERP_SPEC
+#ifdef INTERP_SPEC
                   fsr += fdsrOuter;   fsg += fdsgOuter;   fsb += fdsbOuter;
 #endif
-#if INTERP_ALPHA
+#ifdef INTERP_ALPHA
                   fa += fdaOuter;
 #endif
-#if INTERP_INDEX
+#ifdef INTERP_INDEX
                   fi += fdiOuter;
 #endif
-#if INTERP_INT_ST
+#ifdef INTERP_INT_ST
                   fs += fdsOuter;   ft += fdtOuter;
 #endif
-#if INTERP_STUV
+#ifdef INTERP_STUV
 		  sLeft += dsOuter;
 		  tLeft += dtOuter;
 		  uLeft += duOuter;
 		  vLeft += dvOuter;
 #endif
-#if INTERP_STUV1
+#ifdef INTERP_STUV1
 		  s1Left += ds1Outer;
 		  t1Left += dt1Outer;
 		  u1Left += du1Outer;
@@ -1007,34 +1026,36 @@
                }
                else {
 #ifdef PIXEL_ADDRESS
-                  pRow = (PIXEL_TYPE*) ((GLubyte*)pRow + dPRowInner);
+                  pRow = (PIXEL_TYPE *) ((GLubyte*)pRow + dPRowInner);
 #endif
-#if INTERP_Z
-                  zRow = (GLdepth*) ((GLubyte*)zRow + dZRowInner);
+#ifdef INTERP_Z
+#  ifdef DEPTH_TYPE
+                  zRow = (DEPTH_TYPE *) ((GLubyte*)zRow + dZRowInner);
+#  endif
                   fz += fdzInner;
 #endif
-#if INTERP_RGB
+#ifdef INTERP_RGB
                   fr += fdrInner;   fg += fdgInner;   fb += fdbInner;
 #endif
-#if INTERP_SPEC
+#ifdef INTERP_SPEC
                   fsr += fdsrInner;   fsg += fdsgInner;   fsb += fdsbInner;
 #endif
-#if INTERP_ALPHA
+#ifdef INTERP_ALPHA
                   fa += fdaInner;
 #endif
-#if INTERP_INDEX
+#ifdef INTERP_INDEX
                   fi += fdiInner;
 #endif
-#if INTERP_INT_ST
+#ifdef INTERP_INT_ST
                   fs += fdsInner;   ft += fdtInner;
 #endif
-#if INTERP_STUV
+#ifdef INTERP_STUV
 		  sLeft += dsInner;
 		  tLeft += dtInner;
 		  uLeft += duInner;
 		  vLeft += dvInner;
 #endif
-#if INTERP_STUV1
+#ifdef INTERP_STUV1
 		  s1Left += ds1Inner;
 		  t1Left += dt1Inner;
 		  u1Left += du1Inner;
@@ -1067,3 +1088,7 @@
 
 #undef S_SCALE
 #undef T_SCALE
+
+#undef FixedToDepth
+
+#undef DO_OCCLUSION_TEST
